#!/bin/sh

echo "This script replaces WTFPL v2 headers with GPL v2 headers"
echo "in all CBS files and copies the licence text from the parent"
echo "directory."
echo ""
echo "It is meant to be used from an active Vaucanson repository,"
echo "in the cbs/ directory."
echo "It should work elsewhere, but make backups first."
echo ""
echo "MAKE SURE THE SCRIPT IS STARTED FROM THE CBS ROOT DIR!"
echo ""
echo "Press Enter to continue or Ctrl-C to abort."

read unused

echo ""
echo "Looking for files:"

files=`find . | grep -vE "__backup_$" | grep -vE ".git" | grep -vE "_build"`

rename_cpp()
{
    echo "C++ file"
    res=`cat $file | grep -n -m 16 "// The complete WTFPL license"`
    if [ ! "x$res" = "x" ]; then
	echo "WTFPL header found"
	if [ `expr substr "$res" 1 3` = "11:" ]; then
	    echo "Header in proper location"
	    echo "Rewriting header..."
	    sed -i -f reheader-cpp.sed $file
	else
	    echo "Header not recognized"
	    echo "Not rewriting"
	fi
    else
	echo "WTFPL header not found"
	echo "Not rewriting"
    fi
}

rename_sh()
{
    echo "Script file"
    res=`cat $file | grep -n -m 16 "## The complete WTFPL license"`
    if [ ! "x$res" = "x" ]; then
	echo "WTFPL header found"
	if [ `expr substr "$res" 1 2` = "9:" ]; then
	    echo "Header in proper location"
	    echo "Rewriting header..."
	    sed -i -f reheader-sh.sed $file
	else
	    echo "Header not recognized"
	    echo "Not rewriting"
	fi
    else
	echo "WTFPL header not found"
	echo "Not rewriting"
    fi
}

rename_text()
{
    echo "Text file"
    res=`cat $file | grep -n -m 16 "The complete WTFPL license"`
    if [ ! "x$res" = "x" ]; then
	echo "WTFPL header found"
	if [ `expr substr "$res" 1 2` = "9:" ]; then
	    echo "Header in proper location"
	    echo "Rewriting header..."
	    sed -i -f reheader-text.sed $file
	else
	    echo "Header not recognized"
	    echo "Not rewriting"
	fi
    else
	echo "WTFPL header not found"
	echo "Not rewriting"
    fi
}

for file in $files; do
    if test -f $file; then
	echo $file
	case $file in
	    *.hh)
	        rename_cpp;
		;;
	    *.hxx)
	        rename_cpp;
		;;
	    *.cc)
	        rename_cpp;
		;;
	    *.am)
		rename_sh;
		;;
	    *.ac)
		rename_sh;
		;;
	    *.mk)
		rename_sh;
		;;
	    *README*)
		rename_text;
		;;
	    *)
		echo "Other file."
		res=`grep -n "The complete WTFPL" $file`
		if [ ! "x$res" = "x" ]; then
		    echo "Reference to WTFPL header found:"
		    echo "$res"
		else
		    echo "No reference to WTFPL header found."
		fi
		echo "Not rewriting.";
		;;
	esac
    fi
done
if [ -e "../COPYING" ]; then
    echo "COPYING found in the parent directory."
    echo "Replacing COPYING."
    cp "../COPYING" .
else
    echo "COPYING not found in the parent directory."
    echo "Not copied."
fi
echo ""
echo "Rewriting complete."
echo ""
echo "Remaining references to WTFPL:"
echo ""
for file in $files; do
    grep -nH "WTFPL" $file
done
